"""Import re rich api and saved data"""
import re

from rich import print

from papercli.api import version_groups, version_group_builds
from papercli.save import selected_builds, selected_mc_version, build_name, projects_list


def project_list():
    """List Project's"""
    counter = 0
    for item in projects_list:
        print('(' + str(counter) + ') ' + item)
        counter += 1


def version_group_list(project_number):
    """List Versions"""
    counter = 0
    for item in version_groups(project_number):
        print('(' + str(counter) + ') ' + item)
        counter += 1


def build_list(project_number, mc_version):
    """List Builds"""
    counter = 0
    returnvalue = version_group_builds(project_number, mc_version)
    sorted_val = sorted(returnvalue, key=lambda k: k['build'], reverse=True)
    project = projects_list[project_number]
    for item in sorted_val:
        try:
            print(
                '(' + str(counter) + ') ' + 'Build: ' + str(item['build']) + ' for MC version ' + str(item['version']))
            link = '[link=https://github.com/PaperMC/{poje}/commit/{commit_number}]'.format(
                commit_number=str(item['changes'][0]['commit']), poje=project)
            unformated_summary = str(item['changes'][0]['summary']).format()
            matches = re.search(r"#(\d*)", unformated_summary)
            if matches:
                issueurl = '[link=https://github.com/PaperMC/{poje}/issues/{group}]{urltext}[/link]'.format(
                    group=matches.group(1), urltext=matches.group(), poje=project)
                summery_without_url = re.sub(r"#(\d*)", "", unformated_summary, 1)
                formated_summary = summery_without_url[:matches.start()] + issueurl + summery_without_url[
                                                                                      matches.start():]
            else:
                formated_summary = unformated_summary

            print('     [{commitlink}{commitshort}[/link]] {summary}'.format(commitlink=link,
                                                                             commitshort=str(
                                                                                 item['changes'][0]['commit'])[
                                                                                         :7], summary=formated_summary))
            counter = infosave(counter, item)
        except IndexError:
            print('      there was no info found about ' + str(item['build']))
            counter = infosave(counter, item)


def infosave(counter, item):
    """save info"""
    build_name.append(counter)
    build_name.append(item['downloads']['application']['name'])
    selected_builds.append(counter)
    selected_builds.append(item['build'])
    selected_mc_version.append(counter)
    selected_mc_version.append(item['version'])
    counter += 1
    return counter



"""import argparse sys urllib request and textgen and save"""
import argparse
import sys
import urllib.request

from papercli.api import projects, versions, builds, build_info
from papercli.save import selected_builds, selected_mc_version, projects_list, build_name, versions_list, builds_list
from papercli.textgen import project_list, version_group_list, build_list


def cli_main():
    """Main"""
    try:
        projects()
        cliargs()
    except KeyboardInterrupt:
        # quit
        sys.exit(-1)


def downloade(build, version, p_project, filename, your_filename):
    """Downloade File"""
    url = f'https://api.papermc.io/v2/projects/{p_project}/versions/{version}/builds/{build}/downloads/{filename}'
    print(f'Downloading {p_project} build {build} for MC Version {version}')
    print(f'File will be saved as {your_filename}')
    urllib.request.urlretrieve(url, fr'./{your_filename}')


def cligui(your_filename):
    """Cli GUI"""
    print('Welcome to paper-cli please chose one of the following projects by there corresponding number')
    project_list()
    project = input('Your Input: ')
    print('Next Please select your Target Minecraft Version')
    version_group_list(int(project))
    mcversion = input('Your Input: ')
    build_list(int(project), int(mcversion))
    build_input = input('Your Input: ')
    build = selected_builds[int(build_input) * 2 + 1]
    version = selected_mc_version[int(build_input) * 2 + 1]
    p_project = projects_list[int(project)]
    filename = build_name[int(build_input) * 2 + 1]
    print(f'Do you want to Downloade {p_project} build {build} for MC version {version}?')
    if your_filename is None:
        your_filename = f'{p_project}.jar'
    downloade_question = input('(y/n): ').lower()
    if downloade_question == "y":
        downloade(build, version, p_project, filename, your_filename)
        print('Downloadet')
        sys.exit(0)
    elif downloade_question == "n":
        print('exiting')
        sys.exit(0)
    else:
        print('error invalid anser exeting')
        sys.exit(1)


def cliargs():
    """create cli args"""
    praser = argparse.ArgumentParser(description='Arguments')
    praser.add_argument("--projects", "-p", type=str, choices=projects_list,
                        help=f"select the paper project {projects_list}")
    praser.add_argument("--version", "-v", type=str, help="Select target Minecraft Version")
    praser.add_argument("--build", "-b", type=int, help="Select Build")
    praser.add_argument("--filename", "-f", type=str,
                        help="Select your filename when unset it defaults to {projectname}.jar")
    praser.add_argument("--latest", nargs='?', type=bool, const=True, help="Download latest version")
    args = praser.parse_args()
    your_filename = None
    if args.filename:
        your_filename = args.filename
    if args.projects is None:
        cligui(your_filename)
    else:
        arg_check(args, your_filename)


def arg_paper():
    """return paper id"""
    return 0


def arg_travertine():
    """return travertine id"""
    return 1


def arg_waterfall():
    """return waterfall id"""
    return 2


def fetch_latest(project_id, your_filename):
    """fetch latst build"""
    projects()
    versions(project_id)
    latest_version = len(versions_list) - 1
    builds(project_id, latest_version)
    latest_build = len(builds_list) - 1
    latest_build_info = build_info(project_id, latest_version, latest_build)
    filename = latest_build_info['downloads']['application']['name']
    downloade(builds_list[-1], versions_list[-1], projects_list[project_id], filename, your_filename)


def arg_check(args, your_filename):
    """argument logik"""
    try:
        switch = {
            "paper": arg_paper,
            "travertine": arg_travertine,
            "arg_waterfall": arg_waterfall
        }
        project_id = switch[args.projects]()
        if args.latest:
            fetch_latest(project_id, your_filename)
        elif args.version:
            version = args.version
            if args.build:
                build = args.build
            else:
                build = builds(project_id, versions(project_id).index(version))[-1]
        try:
            sel_version = versions(project_id).index(version)
        except ValueError:
            print("an error ocurred please check your input")
            sys.exit(3)
        try:
            latest_build_info = build_info(project_id, sel_version, build)
            filename = latest_build_info['downloads']['application']['name']
        except KeyError:
            print("an error ocurred please check your input")
            sys.exit(4)
        if your_filename is None:
            your_filename = str(projects_list[project_id]) + '.jar'
        downloade(build, version, projects_list[project_id], filename, your_filename)
    except KeyError:
        print("an error ocurred please check your input")
        sys.exit(1)
    except UnboundLocalError:
        print("an error ocurred please check your input")
        sys.exit(2)



"""import request"""
import requests

from papercli.save import projects_list, version_groups_list, versions_list, builds_list


def api_requests(url):
    """Get request_json"""
    request_resp = requests.get(url)
    request_json = request_resp.json()
    return request_json


def projects():
    """Get projects_list"""
    project_json = api_requests('https://api.papermc.io/v2/projects')
    for item in project_json:
        for data_items in project_json[item]:
            projects_list.append(data_items)
    return projects_list


def version_groups(selected_project):
    """Get version_groups_list"""
    project_info_json = api_requests('https://api.papermc.io/v2/projects/' + projects_list[selected_project])
    for item in project_info_json['version_groups']:
        version_groups_list.append(item)
    return version_groups_list


def version_group_builds(selected_project, groop):
    """get build_infos_version_group_json"""
    build_infos_version_group_json = api_requests(
        'https://api.papermc.io/v2/projects/' + projects_list[selected_project] + '/version_group/' +
        version_groups_list[groop] + '/builds')
    return build_infos_version_group_json['builds'][-15:]


def versions(selected_project):
    """get versions_list"""
    version_json = api_requests('https://api.papermc.io/v2/projects/' + projects_list[selected_project])
    for item in version_json['versions']:
        versions_list.append(item)
    return versions_list


def builds(selected_project, mc_version):
    """Get builds_list"""
    builds_json = api_requests('https://api.papermc.io/v2/projects/' + projects_list[selected_project] + '/versions/' +
                               versions_list[mc_version])
    for item in builds_json['builds']:
        builds_list.append(item)
    return builds_list


def build_info(selected_project, mc_version, build):
    """get build_info_json"""
    build_info_json = api_requests(
        'https://api.papermc.io/v2/projects/' + projects_list[selected_project] + '/versions/' +
        versions_list[mc_version] + '/builds/' + str(build))
    return build_info_json



from setuptools import setup

setup(
    name='paper-cli',
    version='0.2.3',
    packages=['papercli'],
    long_description='tool to easyly get paper',
    install_requires=[
        'requests',
        'rich'
    ],
    entry_points={
        'console_scripts': [
            'paper-cli = papercli.papercli:cli_main',
        ],
    },
)
